/***************************************************************************/
/*                                                                         */
/* File: dxdraw.cpp                                                        */
/* Author: bkenwright@screentoys.net                                       */
/* Date: 10-11-2002                                                        */
/*                                                                         */
/***************************************************************************/

LPDIRECT3D8 g_pD3D = NULL;
LPDIRECT3DDEVICE8 g_pD3DDevice = NULL;

void init(HWND hWnd)
{
    //First of all, create the main D3D object. If it is created successfully we 
    //should get a pointer to an IDirect3D8 interface.
    g_pD3D = Direct3DCreate8(D3D_SDK_VERSION);
    //Get the current display mode
    D3DDISPLAYMODE d3ddm;
    g_pD3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &d3ddm);
    //Create a structure to hold the settings for our device
    D3DPRESENT_PARAMETERS d3dpp; 
    ZeroMemory(&d3dpp, sizeof(d3dpp));
    //Fill the structure. 
    //We want our program to be windowed, and set the back buffer to a format
    //that matches our current display mode
    d3dpp.Windowed = TRUE;
    d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
    d3dpp.BackBufferFormat = d3ddm.Format;
    //For depth buffering (e.g.) the z-buffer
    d3dpp.BackBufferCount=1;
    d3dpp.AutoDepthStencilFormat = D3DFMT_D16;
    d3dpp.EnableAutoDepthStencil = TRUE;
    //Create a Direct3D device
    g_pD3D->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING, &d3dpp, &g_pD3DDevice);
    //Turn on z-buffering
    g_pD3DDevice->SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);

	//Turn on back face culling. This is becuase we want to hide the back of our polygons
    //g_pD3DDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE); //D3DCULL_CCW);
	g_pD3DDevice->SetRenderState(D3DRS_CULLMODE, D3DCULL_CW);
}

void de_init()
{
	g_pD3DDevice->Release();
    g_pD3DDevice = NULL;
    g_pD3D->Release();
    g_pD3D = NULL;
}
void set_camera()
{
    // [1] D3DTS_VIEW
    D3DXMATRIX v;
    g_pD3DDevice->SetTransform(D3DTS_VIEW, D3DXMatrixLookAtLH(&v, &D3DXVECTOR3(0,0,-90), 
                                                                  &D3DXVECTOR3(0,0,0), 
                                                                  &D3DXVECTOR3(0,1,0)));  
    // [2] D3DTS_PROJECTION
    D3DXMATRIX p;
    g_pD3DDevice->SetTransform( D3DTS_PROJECTION, D3DXMatrixPerspectiveFovLH( &p,  D3DX_PI/4,  1.0f,
                                                                                                 1.0f,200.0f));
};

struct my_vertex
{
      FLOAT x, y, z;  // D3DFVF_XYZ
      FLOAT tu, tv;   // D3DFVF_TEX1
};

void Render(CLoadMD2* m)
{
	if(!g_pD3DDevice)return;

	set_camera();

	
	static float angle = 0.0f;
    angle += 0.001f;
	//~~~*~~~~ Create a matrix
	D3DXMATRIX mx;
    //~~~*~~~~ Do somthing to our empty matrix.
    D3DXMatrixRotationY(&mx, angle ); // angle in radians...eg. 1 degree = PI/180
    //~~~*~~~~ Use the matrix! No use having a matrix if we don't use it!
    g_pD3DDevice->SetTransform(D3DTS_WORLD, &mx);
	

	int numFaces = m->m_Md2Header.numTriangles;
	

	UINT my_vertex_description = (D3DFVF_XYZ | D3DFVF_TEX1);
	IDirect3DVertexBuffer8 * DX_vb;
	g_pD3DDevice->CreateVertexBuffer( numFaces * 3 * sizeof(my_vertex), 0, my_vertex_description, D3DPOOL_MANAGED, &DX_vb );

	// Copy our array which is in computer memory over to the directX memory.. using that pointer we
    // just created etc.
    unsigned char *temp_pointer_vb;

    DX_vb->Lock(0,0, &temp_pointer_vb, 0);
	my_vertex* p_mem = (my_vertex*)temp_pointer_vb;

	for(int i=0; i< numFaces; i++)
	{
		for(int c=0; c<3; c++) // 3 sides to each face (e.g. triangle).
		{
			int index = m->m_pTriangles[i].vertexIndex[c];
			p_mem[i*3+c].x = m->m_pFrames[0].pFinalVerts[index].vertex[0];//x
			p_mem[i*3+c].y = m->m_pFrames[0].pFinalVerts[index].vertex[1];//y
			p_mem[i*3+c].z = m->m_pFrames[0].pFinalVerts[index].vertex[2];//z
			
			int texIndex = m->m_pTriangles[i].texIndex[c];
			p_mem[i*3+c].tu = m->m_pTexCoords[ texIndex ].u / 256.0f;
			p_mem[i*3+c].tv = m->m_pTexCoords[ texIndex ].v / 256.0f;
		}
	}

	DX_vb->Unlock();

    // Okay at this point... our graphics card has our vertices stored in it... we've just copied
    // them over :) Some stuff to setup or graphics card!
    //Turn off lighting becuase we are specifying that our vertices have colour
    g_pD3DDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
    g_pD3DDevice->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_SELECTARG1);
	g_pD3DDevice->SetTextureStageState(0,D3DTSS_COLORARG1, D3DTA_TEXTURE);

	// Clear the back buffer to a blue color
	g_pD3DDevice->Clear( 0, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB(0,0,255), 1.0f, 0 );
	
	g_pD3DDevice->BeginScene();

	IDirect3DTexture8* pTexture; 
	D3DXCreateTextureFromFile(g_pD3DDevice, "Pac3D.bmp", &pTexture);

	// Draw our triangle.
	g_pD3DDevice->SetStreamSource(0, DX_vb, sizeof(my_vertex));
	g_pD3DDevice->SetVertexShader(my_vertex_description);
	g_pD3DDevice->SetTexture(0,pTexture);
	g_pD3DDevice->DrawPrimitive(D3DPT_TRIANGLELIST, 0, numFaces);

	g_pD3DDevice->EndScene();
	// After rendering the scene we display it.
	g_pD3DDevice->Present( NULL, NULL, NULL, NULL );
	
	// As where calling the functino over and over again.. we'd be constantly creating new memory
	// without releasing the old if not for this line!
	DX_vb->Release();
}




