/***************************************************************************/
/*                                                                         */
/* File: md2.cpp                                                           */
/* Author: bkenwright@screentoys.net                                       */
/* Date: 10-11-2002                                                        */
/*                                                                         */
/***************************************************************************/

struct stMd2Header
{
	int magic;			// The magic number used to identify the file.
	int version;		// The file version number (must be 8).
	int skinWidth;		// The width in pixels of our image.
	int skinHeight;		// The height in pixels of our image.
	int frameSize;		// The size in bytes the frames are.
	int numSkins;		// The number of skins associated with the model.
	int numVertices;	// The number of vertices.
	int numTexCoords;	// The number of texture coordinates.
	int numTriangles;	// The number of faces (polygons).
	int numGlCommands;	// The number of gl commands.
	int numFrames;		// The number of animated frames.
	int offsetSkins;	// The offset in the file for the skin data.
	int offsetTexCoords;// The offset in the file for the texture data.
	int offsetTriangles;// The offset in the file for the face data.
	int offsetFrames;	// The offset in the file for the frames data.
	int offsetGlCommands;// The offset in the file for the gl commands data.
	int offsetEnd;		// The end of the file offset.
};

// Some structures to hold or read in data in.
struct stMd2Skins
{
	char skinName[64];
};

struct stMd2TexCoords
{
	short u, v;
};

struct stMd2Triangles
{
	short vertexIndex[3];
	short texIndex[3];
};

struct stMd2Vertices
{
	float vertex[3];
	float normal[3];
};
struct stMd2Frames
{
	stMd2Vertices* pFinalVerts;
};

/***************************************************************************/
/*                                                                         */
/* The CLoadMD2 class, yup its name speaks for itself, it loads the 3D     */
/* model data from the .md2 file and then we can access its public data    */
/* variables to use the data.                                              */
/*                                                                         */
/***************************************************************************/

class CLoadMD2
{
public:
	CLoadMD2(){};
	~CLoadMD2(){};
public:
	bool ImportMD2(char* szFileName);
	void Release();

protected:
	void ReadMD2Data();

	FILE* m_fp;
public:
	stMd2Header		m_Md2Header;

	stMd2Skins		*m_pSkins;
	stMd2Triangles	*m_pTriangles;
	stMd2TexCoords  *m_pTexCoords;
	stMd2Frames		*m_pFrames;
};

/***************************************************************************/
/*                                                                         */
/* The action implimentations of our CLoadMD2 class.                       */
/*                                                                         */
/***************************************************************************/
bool CLoadMD2::ImportMD2(char* szFileName)
{
	m_fp = fopen(szFileName, "rb");

	ReadMD2Data();

	fclose(m_fp);
	return true;
}

void CLoadMD2::ReadMD2Data()
{
	fread(&m_Md2Header, 1, sizeof(m_Md2Header), m_fp);	

	// Allocate memory for our data so we can read it in.
	m_pSkins		= new stMd2Skins	[ m_Md2Header.numSkins ];
	m_pTexCoords	= new stMd2TexCoords[ m_Md2Header.numTexCoords ];
	m_pTriangles	= new stMd2Triangles[ m_Md2Header.numTriangles ];
	m_pFrames		= new stMd2Frames	[ m_Md2Header.numFrames ];

	// -1- Seek to the start of our skins name data and read it in.
	fseek(m_fp, m_Md2Header.offsetSkins, SEEK_SET);
	fread(m_pSkins, sizeof(stMd2Skins), m_Md2Header.numSkins, m_fp);

	// -2- Seek to the start of our Texture Coord data and read it in.
	fseek(m_fp, m_Md2Header.offsetTexCoords, SEEK_SET);
	fread(m_pTexCoords, sizeof(stMd2TexCoords), m_Md2Header.numTexCoords, m_fp);

	// -3- Seek to the start of the Triangle(e.g. Faces) data and read that in.
	fseek(m_fp, m_Md2Header.offsetTriangles, SEEK_SET);
	fread(m_pTriangles, sizeof(stMd2Triangles), m_Md2Header.numTriangles, m_fp);

	// -4- Finally lets read in "one" of the frames, the first one.!
	
	struct stAliasVerts
	{
		byte vertex[3]; // an index reference into the location of our vertexs
		byte lightNormalIndex; // in index into which tex coords to use.
	};
	struct stAliasFrame
	{
		float scale[3];
		float translate[3];
		char name[16];
		stAliasVerts aliasVerts[1];
	};
	
	unsigned char largebuffer[50000];
	stAliasFrame* pTempFrame = (stAliasFrame*) largebuffer;

	fseek(m_fp, m_Md2Header.offsetFrames, SEEK_SET);
	fread(pTempFrame, 1, m_Md2Header.frameSize, m_fp); // We have read in all the frame data here (into a temporyary!!..eeEKK)..

	m_pFrames[0].pFinalVerts = new stMd2Vertices[ m_Md2Header.numVertices ];
	
	// CONVERSION!  A few things before we can use our read in values,
	// for some reason the Z and Y need to be swapped, as Z is facing up
	// and Y is facing into the screen.
	// Also our texture coordinates values are between 0 and 256, we just
	// divide them all by 256 which makes them between 0 and 1.

	// Swap Z<->Y
	for(int i=0; i< m_Md2Header.numVertices; i++)
	{
		m_pFrames[0].pFinalVerts[i].vertex[0] = pTempFrame->aliasVerts[i].vertex[0] * pTempFrame->scale[0] 
																	+ pTempFrame->translate[0];      // x
		m_pFrames[0].pFinalVerts[i].vertex[2] = -1*(pTempFrame->aliasVerts[i].vertex[1] * pTempFrame->scale[1] 
																	+ pTempFrame->translate[1]); // z
		m_pFrames[0].pFinalVerts[i].vertex[1] = pTempFrame->aliasVerts[i].vertex[2] * pTempFrame->scale[2] 
																	+ pTempFrame->translate[2];      // y
	}

	// Scale Textures.
	for (int j=0; j< m_Md2Header.numTexCoords; j++)
	{
		// WARNING.. you can't put a decimal number into a short...e.g.
		// you can't put 0.1 into a unsigned short int, it will be changed to 0.
		/*
		m_pTexCoords[j].u = m_pTexCoords[j].u ;// 256; //float(m_Md2Header.skinWidth);
		m_pTexCoords[j].v = m_pTexCoords[j].v ;// 256; //float(m_Md2Header.skinHeight);
		*/
	}

}

void CLoadMD2::Release()
{
	// Tidy up before exiting.
	delete[] m_pFrames[0].pFinalVerts;

	delete[] m_pSkins;
	delete[] m_pTexCoords;
	delete[] m_pTriangles;
	delete[] m_pFrames;
}
